﻿
using System.Collections.Generic;
using System.Text.RegularExpressions;
using UnityEngine;
using UnityEngine.UI;



namespace JayJayWon
{
    public static class BAPluginUI
    {
        private static CustomRowGridlessUIElement topTabMenuOptions;

        private static int currentTopTabSelection = TopUITabs.generalSettings;

        private static BaseUITab currentTabUI;

        private static GeneralUITab generalSettingsUITab;
        private static DesktopUITab desktopSettingsUITab;
        private static VRSettingsUITab vrSettingsUITab;
        private static ResourceSettingsUITab resourceSettingsUITab;

        private static UIDynamicButton generalSettingsUID;
        private static UIDynamicButton VRSettingsUID;
        private static UIDynamicButton desktopSettingsUID;
        private static UIDynamicPopup resourceSettingsUID;

        private static JSONStorableMultiEnumStringChooser resourceSettingsJSESC;

        public static void InitUI()
        {
            topTabMenuOptions = new CustomRowGridlessUIElement(80);

            generalSettingsUITab = new GeneralUITab();
            desktopSettingsUITab = new DesktopUITab();
            vrSettingsUITab = new VRSettingsUITab();
            resourceSettingsUITab = new ResourceSettingsUITab();
            topTabMenuOptions.InitUI(CustomRowWidth.full);

            resourceSettingsJSESC = new JSONStorableMultiEnumStringChooser("resourceType", null, null, topEnumManifestName: ResourceType.enumManifestName, topEnumCallbackFunction: ResourceSettingsTabSelected, mainValCallbackFunction: ResourceSettingsTabSelected);
            resourceSettingsJSESC.label = "Resource\nSettings";

            RePopulateResourceTypeSelector();

            generalSettingsUID = topTabMenuOptions.AddGridlessButtonToRow("Global\nSettings", TopUITabs.generalSettings, TopTabSelected,200,0,0);
            VRSettingsUID = topTabMenuOptions.AddGridlessButtonToRow("VR\nSettings", TopUITabs.vrSettings, TopTabSelected, 200, 200, 0);
            desktopSettingsUID = topTabMenuOptions.AddGridlessButtonToRow("Desktop\nSettings", TopUITabs.desktopSettings, TopTabSelected, 200, 400,0);
            
            resourceSettingsUID = topTabMenuOptions.AddGridlessPopupToRow(resourceSettingsJSESC, null, 450,600,0,800,false);
            resourceSettingsUID.popup.currentValue = "Select...";

            generalSettingsUITab.RefreshGroupUI(1);

            topTabMenuOptions.SetButtonColorActive(currentTopTabSelection);

            currentTabUI = generalSettingsUITab;
        }

        public static void RePopulateResourceTypeSelector()
        {
            ResourceManifest.PopulateResourceTypeSelector(resourceSettingsJSESC);
        }

        public static void ResourceSettingsTabSelected(string value)
        {
            ResourceSettingsTabSelected(ResourceType.presetAtom);
        }
        private static void ResourceSettingsTabSelected(int val)
        {
            if (currentTopTabSelection!= TopUITabs.resourceSettings)
            {
                currentTopTabSelection = TopUITabs.resourceSettings;
                ClearCurrentTopTabsUI();
                
                currentTabUI = resourceSettingsUITab;             
            }
            resourceSettingsUITab.resourceType = val;
            if (val == ResourceType.presetAtom) resourceSettingsUITab.atomPresetResourceType = resourceSettingsJSESC.mainVal;

            if (PC.patreonContentEnabled || val == ResourceType.scene || val == ResourceType.presetAppearance)
            {
                if (val != ResourceType.presetAtom) BrowserAssist.mainBrowserUI.browserFiltersUI.currentResourceType = val;
                else BrowserAssist.mainBrowserUI.browserFiltersUI.currentAtomPresetResourceType = resourceSettingsJSESC.mainVal;
            }
            resourceSettingsUITab.RefreshGroupUI(1);
        }
        private static void TopTabSelected(int tabRef)
        {
            if (tabRef != currentTopTabSelection)
            {
                ClearCurrentTopTabsUI();
                switch (tabRef)
                {
                    case TopUITabs.generalSettings:
                        generalSettingsUITab.RefreshGroupUI(1);
                        currentTabUI = generalSettingsUITab;
                        break;
                    case TopUITabs.vrSettings:
                        vrSettingsUITab.RefreshGroupUI(1);
                        currentTabUI = vrSettingsUITab;
                        break;
                    case TopUITabs.desktopSettings:
                        desktopSettingsUITab.RefreshGroupUI(1);
                        currentTabUI = desktopSettingsUITab;
                        break;

                }
                currentTopTabSelection = tabRef;
            }
        }

        private static void ClearCurrentTopTabsUI()
        {
            currentTabUI.ClearAllGroupsUI();

        }

        public static void CloseAllPopups(UIDynamicPopup exception)
        {
            currentTabUI.CloseAllPopups(exception);
        }
    }

    public class GeneralUITab : BaseUITab
    {
        private CustomRowGridUIElement generalSettingsTabMenuOptions;
        private CustomUIElement generalSettingsCustomUI;
        private CustomUIElement helpTextCustomUI;

        private HashSet<ResourceTypeSettingsRowUI> resourceTypeSettingRows = new HashSet<ResourceTypeSettingsRowUI>();

        private int currentGeneralSettingsTab = GeneralSettingsUITabs.general;

        public GeneralUITab()
        {
            generalSettingsTabMenuOptions = new CustomRowGridUIElement();
            uiGroupCount = 2;

            generalSettingsCustomUI = new CustomUIElement(825);
            helpTextCustomUI = new CustomUIElement(200);
            defaultHelpText = "The Global Settings tab contain settings that impact general behavior of the Browser Assist UI - which are not specific to VR, Desktop or Resource types.";
            helpTextJSS.val = defaultHelpText;
            
        }

        private void GeneralSettingsTabSelected(int tabRef)
        {
            if (tabRef != currentGeneralSettingsTab)
            {
                currentGeneralSettingsTab = tabRef;
                RefreshGroupUI(2);
            }
        }

        protected override void CreateGroupUI(int groupID)
        {
            if (groupID == 1)
            {
                generalSettingsTabMenuOptions.InitUI(CustomRowWidth.full, 4);
                generalSettingsTabMenuOptions.AddButtonToRow("General", GeneralSettingsUITabs.general, GeneralSettingsTabSelected);
                generalSettingsTabMenuOptions.AddButtonToRow("Resource Types", GeneralSettingsUITabs.resources, GeneralSettingsTabSelected);

                RegisterCustomUIElement(groupID, generalSettingsTabMenuOptions);

                generalSettingsTabMenuOptions.SetButtonColorActive(currentGeneralSettingsTab);
            }
            if (groupID == 2)
            {
                generalSettingsCustomUI.InitUI(CustomRowWidth.full);
                RegisterCustomUIElement(2, generalSettingsCustomUI);

                if (currentGeneralSettingsTab == GeneralSettingsUITabs.general) CreateUIGeneralSettings();

                if (currentGeneralSettingsTab == GeneralSettingsUITabs.resources) CreateUIResourceTypeSettings();
            }

                
        }

        private JSONStorableString resourceTypeFilterJSS = new JSONStorableString("resourceTypeFilter", "");
        private GameObject resourceTypesListContentGO;
        private GameObject resourceTypeSettingsBoxGO;

        private void CreateUIResourceTypeSettings()
        {
            if (PC.patreonContentEnabled) defaultHelpText = "The Resource Type tab allows each Resource Type to be concealed. Once a Resource Type has been marked as 'Concealed' it will not longer be selectable in the Browser Filter or Resource Settings dropdowns. This enables less useful resouce types to be ignored and presents a simpler set of choices for the user.";
            else defaultHelpText = "The Resource Type tab allows each Resource Type to be concealed. Once a Resource Type has been marked as 'Concealed' it will not longer be selectable in the Browser Filter or Resource Settings dropdowns. This is useful in Patreons version of the plugin where many different types of resources are supported.";
            helpTextJSS.val = defaultHelpText;

            helpTextCustomUI.InitUI(CustomRowWidth.full);

            var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f);
            helpTextBox.textColor = Color.white;
            helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);

            resourceTypeFilterJSS.val = "";
            var resourceTypeSettingsFilterTextBox = generalSettingsCustomUI.AddTextEntryBox(resourceTypeFilterJSS, 540f, 50f, 30, 0f, 5f, TextAnchor.MiddleLeft, InputField.LineType.SingleLine, "Enter filter text for Resource list...", 0f, 0f, ResourceTypeFilterNameChangeCallback);
            ApplyHelp(resourceTypeSettingsFilterTextBox, "Enter a search string to filter the list of Resource Types and allow quicker selection from a large number of resource types.");
            resourceTypeSettingsBoxGO = generalSettingsCustomUI.CreateVScrollableContent(AnchorPresets.topLeft, 1050f, 770f, new Vector2(0f, -50f), 20f, 3f);
            resourceTypesListContentGO = resourceTypeSettingsBoxGO.transform.Find("scrollableContentViewportGO/vScrollableContentGO").gameObject;
            
            RefreshResourceTypeSettingsList();

            RegisterCustomUIElement(2, helpTextCustomUI);
        }

        private void ResourceTypeFilterNameChangeCallback(string resourceFilter)
        {
            resourceTypeFilterJSS.val = resourceFilter;
            RefreshResourceTypeSettingsList();

        }

        private void ClearResourceTypeSettingsList()
        {
            foreach (var resourceTypeRow in resourceTypeSettingRows)
            {
                resourceTypeRow.ClearUI();
            }
            resourceTypeSettingRows.Clear();
        }

        private string GetResourceTypeDisplayName(int resourceType, string atomPresetResourceType)
        {
            string displayName = EnumManifest.GetDisplayVal(ResourceType.enumManifestName, resourceType);

            if (resourceType == ResourceType.presetAtom)
            {
                displayName = "Preset: "+atomPresetResourceType;
            }

            return displayName;
        }
        public void RefreshResourceTypeSettingsList()
        {           
            ClearResourceTypeSettingsList();

            string filter = resourceTypeFilterJSS.val.ToLower();

            List<int> resourceTypes = EnumManifest.GetEnumsList(ResourceType.enumManifestName);

            bool disableConcealResourceTypes= false;
            if (Settings.concealedResourceTypeCount >= resourceTypes.Count - 2) disableConcealResourceTypes = true;

            if (!PC.patreonContentEnabled)
            {
                if (Settings.IsResourceConcealed(ResourceType.scene,"") || Settings.IsResourceConcealed(ResourceType.presetAppearance, "")) disableConcealResourceTypes = true;
            }

            foreach (int resourceType in resourceTypes)
            {
                if (PC.patreonContentEnabled || resourceType == ResourceType.scene || resourceType == ResourceType.presetAppearance)
                {
                    if (resourceType != ResourceType.presetAtom)
                    {
                        string atomPresetResourceType = "";
                        if (ResourceType.GetResourceCategory(resourceType) == ResourceCategory.preset) atomPresetResourceType = "Person";

                        if (filter == "" || GetResourceTypeDisplayName(resourceType, atomPresetResourceType).ToLower().Contains(filter))
                        {
                            var tagSettingRow = new ResourceTypeSettingsRowUI(resourceTypesListContentGO, resourceType, atomPresetResourceType, this, disableConcealResourceTypes);
                            resourceTypeSettingRows.Add(tagSettingRow);
                        }

                    }
                    else
                    {
                        foreach (string atomPresetResourceType in ResourceManifest.existingAtomPresetResourceTypes)
                        {
                            if (filter == "" || GetResourceTypeDisplayName(resourceType, atomPresetResourceType).ToLower().Contains(filter))
                            {
                                var tagSettingRow = new ResourceTypeSettingsRowUI(resourceTypesListContentGO, resourceType, atomPresetResourceType, this, false);
                                resourceTypeSettingRows.Add(tagSettingRow);
                            }

                        }
                    }
                }
                

            }
        }

        private void CreateUIGeneralSettings()
        {
            var launchBAToggle = generalSettingsCustomUI.AddToggle(Settings.enableTopScreenLaunchButtonJSB, 1020, 50, 0, 0, "Enable BrowserAssist Launch button on VAM Top Menu");
            var launchButtonPlacementPopup = generalSettingsCustomUI.AddPopup(Settings.launchButtonPlacementJSESC, null, 1020, 80, 0, -60, labelProportion: 0.7f);
            var defaultResourceTypePopup = generalSettingsCustomUI.AddPopup(Settings.defaultResourceTypeJMESC, null, 1020, 80, 0, -150, panelHeight: 750f, labelProportion: 0.7f);
            var resourceNameSearchModePopup = generalSettingsCustomUI.AddPopup(Settings.resourceNameSearchModeJSESC, null, 1020, 80, 0, -240, labelProportion: 0.7f);
            var defaultTargetPersonType = generalSettingsCustomUI.AddPopup(Settings.defaultTargetPersonTypeJSE, null, 1020, 80, 0, -330, labelProportion: 0.7f);
            var creatorFilterSortOrder = generalSettingsCustomUI.AddPopup(Settings.creatorFilterOrderJSEC, null, 1020, 80, 0, -420, labelProportion: 0.7f);

            
            var enableBulkDelete = generalSettingsCustomUI.AddToggle(Settings.enableBulkDeleteJSB, 1020, 50, 0, -510, "Enable bulk delete of multi selected resources");

            var disableMetaPurgeToggle = generalSettingsCustomUI.AddToggle(Settings.disableMetaPurgeOnMissingVARJSB, 1020, 50, 0, -570, "Disable purge of meta data on missing VAR");

            var rescanVARsButton = generalSettingsCustomUI.AddButton(1020, 50, 0, -630, "Rescan all VAR packages and update contents", delegate
            {
                VARPackageManifest.RescanAllVARResource();
            });

            helpTextCustomUI.InitUI(CustomRowWidth.full);

            var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f);
            helpTextBox.textColor = Color.white;
            ApplyHelp(launchBAToggle, "When this setting is enabled, a Browser Assist launch button will be displayed embeded into the VAM 'Top Menu' UI. The 'Top Menu' is the UI which is displayed when VAM first starts. It can also be accessed by pressing the star icon on the tool bar in the main VAM UI.");

            ApplyHelp(launchButtonPlacementPopup, "The launch button placement setting determines where the main BrowserAssist launch button is placed on the VAM UI toolbar. Options allow it to be placed over the VAM Scene Browser button (effectively replacing it) or to the left or right of the toolbar.");

            ApplyHelp(defaultResourceTypePopup, "This setting determines which Resource Type (e.g. Scene or Appearance preset) is selected by default when Browser Assist first starts.");

            ApplyHelp(resourceNameSearchModePopup, "The 'Resource Filter Name Search Mode' setting determines the behaviour of the resource name search filter in Browser Assist. The 'Starts With' option will filter to resources where the resource name starts with the search text. The two 'Contains' option will filter to resource names that contain the search text. 'Contains (Full Path)' will search for any matching text in the entire path name including the package name and folders.");

            ApplyHelp(defaultTargetPersonType, "This setting determines the default person gender when selecting a resource that has a person as it's target (e.g. Appearance Presets will select either a male or female Atom where available in the current scene). The setting can be overridden by setting the 'male' or 'female' tag on individual resources.");

            ApplyHelp(creatorFilterSortOrder, "This setting determines the order which VAR Creators are listed in the Creator filter dropdown. This can either be alphabetical or by the quantity  of resources within VARs packaged by that Creator.");

            ApplyHelp(enableBulkDelete, "This setting determines whether a button is available on the BrowserAssist Action tab to delete multiple resources (and potentially their containing VARs) when many resources are selected using the multi-select feature. It can potentially delete a large number of files in a few clicks with no easy method to recover the files. As such, the feature should be used with caution.");

            ApplyHelp(disableMetaPurgeToggle, "Determines whether meta data (e.g. Tags, Hidden) are removed when a resource's VAR package is not detected on BrowserAssist startup. This is intended to be used with VARManager tools to ensure meta data is persisted when VAR packages are held away from the AddonFolders package. When Packages are deleted using the BrowserAssist delete button, meta data associated with that VAR will be purged even if this setting is enabled. ");

            ApplyHelp(rescanVARsButton, "Performs a rescan of all VARs in the AddonPackages folder and identifies any new or removed resources inside the VAR pacakages. This may take some time, but only needs to be performed if any VAR packages have been unpacked, modified (i.e. resources added or removed) and re-packed.");



            RegisterCustomUIElement(2, helpTextCustomUI);

            helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);


        }
    }

    public class ResourceTypeSettingsRowUI
    {
        private GameObject rowGO;
        private GeneralUITab parentUI;

        private int resourceType;
        private string resourceTypeName;
        private string atomPresetResourceType;

        private GameObject resourceTypeTextBoxGO;

        private UIDynamicToggle resourceConcealToggle;

        private JSONStorableBool concealedJSB;

        private bool disableConcealResourceTypes;

        public ResourceTypeSettingsRowUI(GameObject parentGO, int resourceType, string atomPresetResourceType, GeneralUITab parentUI, bool disableConcealResourceTypes)
        {
            rowGO = UI.AddChildGOImage(parentGO, new Color(0f,0f,0f,0f), AnchorPresets.hStretchTop, 1030f, 60f, Vector2.zero);

            this.resourceType = resourceType;
            this.atomPresetResourceType = atomPresetResourceType;
            this.parentUI = parentUI;
            this.disableConcealResourceTypes = disableConcealResourceTypes;

            concealedJSB = new JSONStorableBool("concealed", Settings.IsResourceConcealed(resourceType,atomPresetResourceType), ConcealCallback);

            if (resourceType != ResourceType.presetAtom) resourceTypeName = EnumManifest.GetDisplayVal(ResourceType.enumManifestName, resourceType);
            else resourceTypeName = "Preset: " + atomPresetResourceType;

            CreateUI();
        }

        private void ConcealCallback(bool value)
        {
            List<int> resourceTypes = EnumManifest.GetEnumsList(ResourceType.enumManifestName);
            if (value)
            {
                Settings.SetConcealed(resourceType, atomPresetResourceType);  

                if (Settings.concealedResourceTypeCount >= resourceTypes.Count - 2 || !PC.patreonContentEnabled) parentUI.RefreshResourceTypeSettingsList();
            }
            else
            {
                Settings.UnsetConcealed(resourceType, atomPresetResourceType);
                if (Settings.concealedResourceTypeCount >= resourceTypes.Count - 3 || !PC.patreonContentEnabled) parentUI.RefreshResourceTypeSettingsList();
            }
            Settings.SaveSettings();

            BrowserAssist.mainBrowserUI.browserFiltersUI.RePopulateResourceTypeSelector();
            BAPluginUI.RePopulateResourceTypeSelector();
            Settings.RePopulateResourceTypeSelector();
        }

        public void ClearUI()
        {
            if (resourceTypeTextBoxGO != null) GameObject.Destroy(resourceTypeTextBoxGO);
            if (resourceConcealToggle != null) GameObject.Destroy(resourceConcealToggle.gameObject);
            GameObject.Destroy(rowGO);
        }

        private void RefreshUI()
        {
            if (resourceTypeTextBoxGO != null) GameObject.Destroy(resourceTypeTextBoxGO);
            if (resourceConcealToggle != null) GameObject.Destroy(resourceConcealToggle.gameObject);

            CreateUI();
        }

        private void CreateUI()
        {

            resourceTypeTextBoxGO = UI.CreateBasicTextBox(rowGO, resourceTypeName, 540f, 60f, 30, 0f, 0f, AnchorPresets.middleLeft, true, 18, HorizontalWrapMode.Overflow, rescaleText: false);
            parentUI.ApplyHelp(resourceTypeTextBoxGO, "This is the name of the Resource Type being configured.");

            resourceConcealToggle = UI.CreateToggle(rowGO, concealedJSB, 460f, 60f, "Conceal Resource Type", 26, 550f, 0f, AnchorPresets.middleLeft, 40f, rescaleText: false);

            if (disableConcealResourceTypes && !concealedJSB.val) resourceConcealToggle.toggle.interactable= false;

            if (resourceConcealToggle.toggle.interactable) 
            parentUI.ApplyHelp(resourceConcealToggle, "Conceal this Resource Type from the list of available filters. Once activated, the Browser Filter will not offer resources of the specified type as an option within BrowserAssist.");
            else parentUI.ApplyHelp(resourceConcealToggle, "Concealing all core Resource Types is not supported. Reveal a different core resource type if you want to conceal this one. A core resource type is defined as anything other than the Non-person Atom Presets.");


        }


    }



    public class DesktopUITab : BaseUITab
    {
        private CustomRowGridUIElement desktopSettingsTabMenuOptions;
        private CustomUIElement desktopSettingsCustomUI;
        private CustomUIElement helpTextCustomUI;

        private int currentDesktopSettingsTab = DesktopSettingsUITabs.general;
        public DesktopUITab()
        {
            desktopSettingsTabMenuOptions = new CustomRowGridUIElement();
            uiGroupCount = 2;

            desktopSettingsCustomUI = new CustomUIElement(825);
            helpTextCustomUI = new CustomUIElement(200);
            defaultHelpText = "The Desktop Settings tab contain settings that impact behavior of the Browser Assist UI when in Desktop mode (as opposed to VR mode)";
            helpTextJSS.val = defaultHelpText;


        }

        private void DesktopSettingsTabSelected(int tabRef)
        {
            if (tabRef != currentDesktopSettingsTab)
            {
                currentDesktopSettingsTab = tabRef;
                RefreshGroupUI(2);
            }
        }

        private void CreateUIGeneralSettings()
        {
            var uiScaleSlider = desktopSettingsCustomUI.AddSlider(Settings.uiScaleJSF, 1020, 120, 0, 0);
            uiScaleSlider.valueFormat = "F2";
            uiScaleSlider.ConfigureQuickButtons(-0.1f, -0.25f, -0.5f, -1f, 0.1f, 0.25f, 0.5f, 1f);


            var maxButtonsSlider = desktopSettingsCustomUI.AddSlider(Settings.maxButtonsPerPageDesktopJSF, 1020, 120, 0, -130);
            maxButtonsSlider.labelText.text = "Maximum number of Resources per Browser page";
            maxButtonsSlider.valueFormat = "F0";
            maxButtonsSlider.ConfigureQuickButtons(-10f, -50f, -100f, -1000f, 10f, 50f, 100f, 1000f);

            helpTextCustomUI.InitUI(CustomRowWidth.full);

            var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f);
            helpTextBox.textColor = Color.white;
            helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);


            ApplyHelp(uiScaleSlider, "The 'Desktop UI Scale' setting determines the scale of Browser Assist UI when in Desktop mode. Higher scales numbers make the UI larger and lower scale makes the UI smaller. ");

            ApplyHelp(maxButtonsSlider, "This setting determines the maximum number of resources icons that will be displayed on a single page of browser results, whilst in Desktop mode.");


            RegisterCustomUIElement(2, helpTextCustomUI);
        }

        private void CreateUINewAtomPositionSettings()
        {

            var xPosSlider = desktopSettingsCustomUI.AddSlider(Settings.desktopDefaultNewAtomPosRot.positionXJSF, 500, 120, 0, 0);
            xPosSlider.valueFormat = "F3";
            xPosSlider.label = "New Atom: Default Position X";

            var yPosSlider = desktopSettingsCustomUI.AddSlider(Settings.desktopDefaultNewAtomPosRot.positionYJSF, 500, 120, 0, -120);
            yPosSlider.valueFormat = "F3";
            yPosSlider.label = "New Atom: Default Position Y";

            var zPosSlider = desktopSettingsCustomUI.AddSlider(Settings.desktopDefaultNewAtomPosRot.positionZJSF, 500, 120, 0, -240);
            zPosSlider.valueFormat = "F3";
            zPosSlider.label = "New Atom: Default Position Z";

            var xRotSlider = desktopSettingsCustomUI.AddSlider(Settings.desktopDefaultNewAtomPosRot.rotationXJSF, 500, 120, 510, 0);
            xRotSlider.valueFormat = "F3";
            xRotSlider.label = "New Atom: Default Rotation X";

            var yRotSlider = desktopSettingsCustomUI.AddSlider(Settings.desktopDefaultNewAtomPosRot.rotationYJSF, 500, 120, 510, -120);
            yRotSlider.valueFormat = "F3";
            yRotSlider.label = "New Atom: Default Rotation Y";

            var zRotSlider = desktopSettingsCustomUI.AddSlider(Settings.desktopDefaultNewAtomPosRot.rotationZJSF, 500, 120, 510, -240);
            zRotSlider.valueFormat = "F3";
            zRotSlider.label = "New Atom: Default Rotation Z";

            var xPosAbsRelPopup = desktopSettingsCustomUI.AddPopup(Settings.desktopDefaultNewAtomPosRot.posXRelAbsJSE, null, 500, 60, 0, -360, 140, 60, "X Pos: Relative or Absolute", 0.8f);

            var yPosAbsRelPopup = desktopSettingsCustomUI.AddPopup(Settings.desktopDefaultNewAtomPosRot.posYRelAbsJSE, null, 500, 60, 0, -420, 140, 60, "Y Pos: Relative or Absolute", 0.8f);

            var zPosAbsRelPopup = desktopSettingsCustomUI.AddPopup(Settings.desktopDefaultNewAtomPosRot.posZRelAbsJSE, null, 500, 60, 0, -480, 140, 60, "Z Pos: Relative or Absolute", 0.8f);

            var xRotAbsRelPopup = desktopSettingsCustomUI.AddPopup(Settings.desktopDefaultNewAtomPosRot.rotXRelAbsJSE, null, 500, 60, 510, -360, 140, 60, "X Rot: Relative or Absolute", 0.8f);

            var yRotAbsRelPopup = desktopSettingsCustomUI.AddPopup(Settings.desktopDefaultNewAtomPosRot.rotYRelAbsJSE, null, 500, 60, 510, -420, 140, 60, "Y Rot: Relative or Absolute", 0.8f);

            var zRotAbsRelPopup = desktopSettingsCustomUI.AddPopup(Settings.desktopDefaultNewAtomPosRot.rotZRelAbsJSE, null, 500, 60, 510, -480, 140, 60, "Z Rot: Relative or Absolute", 0.8f);


            helpTextCustomUI.InitUI(CustomRowWidth.full);

            var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f);
            helpTextBox.textColor = Color.white;
            helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);


            ApplyHelp(xPosSlider, "The 'New Atom Default Position X' setting determines the default X coordinate for new atoms created by Browser Assist. The position can be absolute or relative to the desktop view point. If the position is relative, the X coordinate will offset the atom position to the left or right relative to the direction the user is facing.");

            ApplyHelp(yPosSlider, "The 'New Atom Default Position Y' setting determines the default Y coordinate for new atoms created by Browser Assist. The position can be absolute or relative to the desktop view point. If the position is relative, the Y coordinate will offset the atom position to the up or down relative to the direction the user is facing.");

            ApplyHelp(zPosSlider, "The 'New Atom Default Position Z' setting determines the default Z coordinate for new atoms created by Browser Assist. The position can be absolute or relative to the desktop view point. If the position is relative, the Z coordinate will offset the atom position to the further away or closer relative to the users view point.");

            ApplyHelp(xRotSlider, "The 'New Atom Default Rotation X' setting determines the default X rotation (Pitch) for new atoms created by Browser Assist. The rotation can be absolute or relative to the desktop view point. If the rotation is relative, the X rotation will offset the atom's pitch relative to the direction the user is facing.");

            ApplyHelp(yRotSlider, "The 'New Atom Default Rotation Y' setting determines the default Y rotation (Yaw) for new atoms created by Browser Assist. The rotation can be absolute or relative to the desktop view point. If the rotation is relative, the Y rotation will offset the atom's yaw relative to the direction the user is facing.");

            ApplyHelp(zRotSlider, "The 'New Atom Default Rotation Z' setting determines the default Z rotation (Roll) for new atoms created by Browser Assist. The rotation can be absolute or relative to the desktop view point. If the rotation is relative, the Z rotation will offset the atom's roll relative to the direction the user is facing.");

            ApplyHelp(xPosAbsRelPopup, "The 'X Pos: Relative or Absolute' setting determines whether default X position for new atoms (created by Browser Assist) is absolute or relative to the desktop view point.");

            ApplyHelp(yPosAbsRelPopup, "The 'Y Pos: Relative or Absolute' setting determines whether default Y position for new atoms (created by Browser Assist) is absolute or relative to the desktop view point.");

            ApplyHelp(zPosAbsRelPopup, "The 'Z Pos: Relative or Absolute' setting determines whether default Z position for new atoms (created by Browser Assist) is absolute or relative to the desktop view point.");

            ApplyHelp(xRotAbsRelPopup, "The 'X Rot: Relative or Absolute' setting determines whether default X rotation (Pitch) for new atoms (created by Browser Assist) is absolute or relative to the desktop view point.");

            ApplyHelp(yRotAbsRelPopup, "The 'Y Rot: Relative or Absolute' setting determines whether default Y rotation (Yaw) for new atoms (created by Browser Assist) is absolute or relative to the desktop view point.");

            ApplyHelp(zRotAbsRelPopup, "The 'Z Rot: Relative or Absolute' setting determines whether default Z rotation (Roll) for new atoms (created by Browser Assist) is absolute or relative to the desktop view point.");

            RegisterCustomUIElement(2, helpTextCustomUI);
        }

        protected override void CreateGroupUI(int groupID)
        {
            if (groupID == 1)
            {
                desktopSettingsTabMenuOptions.InitUI(CustomRowWidth.full, 4);
                desktopSettingsTabMenuOptions.AddButtonToRow("General", GeneralSettingsUITabs.general, DesktopSettingsTabSelected);
                desktopSettingsTabMenuOptions.AddButtonToRow("New Atom Position", GeneralSettingsUITabs.resources, DesktopSettingsTabSelected);

                RegisterCustomUIElement(groupID, desktopSettingsTabMenuOptions);

                desktopSettingsTabMenuOptions.SetButtonColorActive(currentDesktopSettingsTab);
            }
            if (groupID == 2)
            {
                desktopSettingsCustomUI.InitUI(CustomRowWidth.full);
                RegisterCustomUIElement(2, desktopSettingsCustomUI);

                if (currentDesktopSettingsTab == DesktopSettingsUITabs.general) CreateUIGeneralSettings();

                if (currentDesktopSettingsTab == DesktopSettingsUITabs.newAtomPosition) CreateUINewAtomPositionSettings();
            }

        }
    }

    public class VRSettingsUITab : BaseUITab
    {
        private CustomRowGridUIElement desktopSettingsTabMenuOptions;
        private CustomUIElement vrSettingsCustomUI;
        private CustomUIElement helpTextCustomUI;

        private int currentDesktopSettingsTab = DesktopSettingsUITabs.general;
        public VRSettingsUITab()
        {
            desktopSettingsTabMenuOptions = new CustomRowGridUIElement();
            uiGroupCount = 2;

            vrSettingsCustomUI = new CustomUIElement(825);
            helpTextCustomUI = new CustomUIElement(200);
            defaultHelpText = "The VR Settings tab contain settings that impact behavior of the Browser Assist UI when in VR mode (as opposed to Desktop mode)";
            helpTextJSS.val = defaultHelpText;
        }

        private void DesktopSettingsTabSelected(int tabRef)
        {
            if (tabRef != currentDesktopSettingsTab)
            {
                currentDesktopSettingsTab = tabRef;
                RefreshGroupUI(2);
            }
        }

        private void CreateUIGeneralSettings()
        {
            var dockVAMUIToggle = vrSettingsCustomUI.AddToggle(Settings.dockToVAMUIJSB, 500, 60, 0, 0);
            dockVAMUIToggle.label = "Dock BrowserAssist to VAM UI";

            var maxButtonsSlider = vrSettingsCustomUI.AddSlider(Settings.maxButtonsPerPageVRJSF, 1020, 120, 0, -70);
            maxButtonsSlider.labelText.text = "Maximum number of Resources per Browser page";
            maxButtonsSlider.valueFormat = "F0";
            maxButtonsSlider.ConfigureQuickButtons(-10f, -50f, -100f, -1000f, 10f, 50f, 100f, 1000f);

            helpTextCustomUI.InitUI(CustomRowWidth.full);

            var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f);
            helpTextBox.textColor = Color.white;
            helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);

            ApplyHelp(dockVAMUIToggle, "This setting determines if Browser Assist window is docked to VAM Main UI when in VR. If enabled, the Browser Assist window will behave simlarly to other settings available in VAM UI and can be repostioned using the VR controller B button.");

            ApplyHelp(maxButtonsSlider, "This setting determines the maximum number of resources icons that will be displayed on a single page of browser results, whilst in VR mode.");

            RegisterCustomUIElement(2, helpTextCustomUI);
        }

        private void CreateUINewAtomPositionSettings()
        {
            var xPosSlider = vrSettingsCustomUI.AddSlider(Settings.vrDefaultNewAtomPosRot.positionXJSF, 500, 120, 0, 0);
            xPosSlider.valueFormat = "F3";
            xPosSlider.label = "New Atom: Default Position X";

            var yPosSlider = vrSettingsCustomUI.AddSlider(Settings.vrDefaultNewAtomPosRot.positionYJSF, 500, 120, 0, -120);
            yPosSlider.valueFormat = "F3";
            yPosSlider.label = "New Atom: Default Position Y";

            var zPosSlider = vrSettingsCustomUI.AddSlider(Settings.vrDefaultNewAtomPosRot.positionZJSF, 500, 120, 0, -240);
            zPosSlider.valueFormat = "F3";
            zPosSlider.label = "New Atom: Default Position Z";

            var xRotSlider = vrSettingsCustomUI.AddSlider(Settings.vrDefaultNewAtomPosRot.rotationXJSF, 500, 120, 510, 0);
            xRotSlider.valueFormat = "F3";
            xRotSlider.label = "New Atom: Default Rotation X";

            var yRotSlider = vrSettingsCustomUI.AddSlider(Settings.vrDefaultNewAtomPosRot.rotationYJSF, 500, 120, 510, -120);
            yRotSlider.valueFormat = "F3";
            yRotSlider.label = "New Atom: Default Rotation Y";

            var zRotSlider = vrSettingsCustomUI.AddSlider(Settings.vrDefaultNewAtomPosRot.rotationZJSF, 500, 120, 510, -240);
            zRotSlider.valueFormat = "F3";
            zRotSlider.label = "New Atom: Default Rotation Z";

            var xPosAbsRelPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.posXRelAbsJSE, null, 500, 60, 0, -360, 140, 60, "X Pos: Relative or Absolute", 0.8f);

            var yPosAbsRelPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.posYRelAbsJSE, null, 500, 60, 0, -420, 140, 60, "Y Pos: Relative or Absolute", 0.8f);

            var zPosAbsRelPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.posZRelAbsJSE, null, 500, 60, 0, -480, 140, 60, "Z Pos: Relative or Absolute", 0.8f);

            var xRotAbsRelPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.rotXRelAbsJSE, null, 500, 60, 510, -360, 140, 60, "X Rot: Relative or Absolute", 0.8f);

            var yRotAbsRelPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.rotYRelAbsJSE, null, 500, 60, 510, -420, 140, 60, "Y Rot: Relative or Absolute", 0.8f);

            var zRotAbsRelPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.rotZRelAbsJSE, null, 500, 60, 510, -480, 140, 60, "Z Rot: Relative or Absolute", 0.8f);

            var relAtomPopup = vrSettingsCustomUI.AddPopup(Settings.vrDefaultNewAtomPosRot.relativeAtomNameJSME, null, 500, 60, 510, -550, 200, 60, "Relative to:", 0.6f);

            helpTextCustomUI.InitUI(CustomRowWidth.full);

            var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f);
            helpTextBox.textColor = Color.white;
            helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);

            ApplyHelp(xPosSlider, "The 'New Atom Default Position X' setting determines the default X coordinate for new atoms created by Browser Assist. The position can be absolute or relative to the VR headset or VR controllers. If the position is relative, the X coordinate will offset the atom position to the left or right relative to the direction the user is facing.");

            ApplyHelp(yPosSlider, "The 'New Atom Default Position Y' setting determines the default Y coordinate for new atoms created by Browser Assist. The position can be absolute or relative to the VR headset or VR controllers. If the position is relative, the Y coordinate will offset the atom position to the up or down relative to the direction the user is facing.");

            ApplyHelp(zPosSlider, "The 'New Atom Default Position Z' setting determines the default Z coordinate for new atoms created by Browser Assist. The position can be absolute or relative to the VR headset or VR controllers. If the position is relative, the Z coordinate will offset the atom position to the further away or closer relative to the users view point.");

            ApplyHelp(xRotSlider, "The 'New Atom Default Rotation X' setting determines the default X rotation (Pitch) for new atoms created by Browser Assist. The rotation can be absolute or relative to the VR headset or VR controllers. If the rotation is relative, the X rotation will offset the atom's pitch relative to the direction the user is facing.");

            ApplyHelp(yRotSlider, "The 'New Atom Default Rotation Y' setting determines the default Y rotation (Yaw) for new atoms created by Browser Assist. The rotation can be absolute or relative to the VR headset or VR controllers. If the rotation is relative, the Y rotation will offset the atom's yaw relative to the direction the user is facing.");

            ApplyHelp(zRotSlider, "The 'New Atom Default Rotation Z' setting determines the default Z rotation (Roll) for new atoms created by Browser Assist. The rotation can be absolute or relative to the VR headset or VR controllers. If the rotation is relative, the Z rotation will offset the atom's roll relative to the direction the user is facing.");

            ApplyHelp(xPosAbsRelPopup, "The 'X Pos: Relative or Absolute' setting determines whether default X position for new atoms (created by Browser Assist) is absolute or relative to the VR headset or VR controllers.");

            ApplyHelp(yPosAbsRelPopup, "The 'Y Pos: Relative or Absolute' setting determines whether default Y position for new atoms (created by Browser Assist) is absolute or relative to the VR headset or VR controllers.");

            ApplyHelp(zPosAbsRelPopup, "The 'Z Pos: Relative or Absolute' setting determines whether default Z position for new atoms (created by Browser Assist) is absolute or relative to the VR headset or VR controllers.");

            ApplyHelp(xRotAbsRelPopup, "The 'X Rot: Relative or Absolute' setting determines whether default X rotation (Pitch) for new atoms (created by Browser Assist) is absolute or relative to the VR headset or VR controllers.");

            ApplyHelp(yRotAbsRelPopup, "The 'Y Rot: Relative or Absolute' setting determines whether default Y rotation (Yaw) for new atoms (created by Browser Assist) is absolute or relative to the VR headset or VR controllers.");

            ApplyHelp(zRotAbsRelPopup, "The 'Z Rot: Relative or Absolute' setting determines whether default Z rotation (Roll) for new atoms (created by Browser Assist) is absolute or relative to the VR headset or VR controllers.");

            ApplyHelp(relAtomPopup, "If any of the atoms positions or rotations are set as 'Relative', then this setting determines whether that position or rotation is relative to the VR Headset, VR Left Controller or the VR right Controller.");

            RegisterCustomUIElement(2, helpTextCustomUI);

        }
        protected override void CreateGroupUI(int groupID)
        {
            if (groupID == 1)
            {
                desktopSettingsTabMenuOptions.InitUI(CustomRowWidth.full, 4);
                desktopSettingsTabMenuOptions.AddButtonToRow("General", GeneralSettingsUITabs.general, DesktopSettingsTabSelected);
                desktopSettingsTabMenuOptions.AddButtonToRow("New Atom Position", GeneralSettingsUITabs.resources, DesktopSettingsTabSelected);

                RegisterCustomUIElement(groupID, desktopSettingsTabMenuOptions);

                desktopSettingsTabMenuOptions.SetButtonColorActive(currentDesktopSettingsTab);
            }
            if (groupID == 2)
            {
                vrSettingsCustomUI.InitUI(CustomRowWidth.full);
                RegisterCustomUIElement(2, vrSettingsCustomUI);

                if (currentDesktopSettingsTab == DesktopSettingsUITabs.general) CreateUIGeneralSettings();

                if (currentDesktopSettingsTab == DesktopSettingsUITabs.newAtomPosition) CreateUINewAtomPositionSettings();
            }

            

            
        }
    }


}
